#!/bin/sh
#
# Fire up 3 to 10 JVMs (all but the last in an xterm).  Runs ANTS
# ConfigurationManager on each node with $CONFIG_FILE as the config
# file.  The Nodes are given the IPs $NODE1_IP, $NODE2_IP, $NODE3_IP ...
#  
# Copyright (c) 2001 University of Utah and the Flux Group.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this file
# for any purpose with or without restriction is hereby granted.
#

# Make sure required
if test -z "$CONFIG_FILE" -o -z "$NODE1_IP" -o -z "$NODE2_IP"; then
    echo "Don't run this directly."
    exit 1
fi

# Default to zero nodes
NODECT=${NODECT:-0}

if test "$NODECT" -lt 2 -o "$NODECT" -gt 10; then
    echo "./nodesetup.sh Requires between 2 and 10 nodes be configured."
    exit 1
fi

# Source defaultEnv scripts if it exists
if test -f ./defaultEnv; then
    . ./defaultEnv
fi

# Define vars from defaultEnv values, or real defaults.
JVM=${JVM:-${JVM_DEFAULT:-java}}
XTERM=${XTERM:-${XTERM_DEFAULT:-"xterm -sb -sl 4000"}}
SH=${SH:-"sh"}
PS=${PS:-${PS_DEFAULT:-"ps xwww"}}
KILL=${KILL:=${KILL_DEFAULT:-"/bin/kill"}}

## Default JVMFLAGS to ''
#JVMFLAGS=

## Double-check that the JVM works.
echo "Trying $JVM"
$JVM -version > /dev/null 2>&1
if test $? -ne 0; then
    echo "ERROR: The JVM '$JVM' either isn't in your path or doesn't work.  Please fix it."
    echo "(Tried '$JVM -version'.)"
    exit 12
fi

# convenience
ANTS_CONFIGMGR="edu.utah.janos.nodeos.Main ants.core.ConfigurationManager"

# Node 10
if test $NODECT -ge 10; then
    echo "Starting node 10 ($NODE10_IP): $NODE10_LABEL"
    $XTERM -T "($NODE10_IP) $NODE10_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE10_IP && cat || cat" &
    J10=$!
    sleep 1
fi

# Node 9
if test $NODECT -ge 9; then
    echo "Starting node 9 ($NODE9_IP): $NODE9_LABEL"
    $XTERM -T "($NODE9_IP) $NODE9_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE9_IP && cat || cat" &
    J9=$!
    sleep 1
fi

# Node 8
if test $NODECT -ge 8; then
    echo "Starting node 8 ($NODE8_IP): $NODE8_LABEL"
    $XTERM -T "($NODE8_IP) $NODE8_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE8_IP && cat || cat" &
    J8=$!
    sleep 1
fi

# Node 7
if test $NODECT -ge 7; then
    echo "Starting node 7 ($NODE7_IP): $NODE7_LABEL"
    $XTERM -T "($NODE7_IP) $NODE7_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE7_IP && cat || cat" &
    J7=$!
    sleep 1
fi

# Node 6
if test $NODECT -ge 6; then
    echo "Starting node 6 ($NODE6_IP): $NODE6_LABEL"
    $XTERM -T "($NODE6_IP) $NODE6_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE6_IP && cat || cat" &
    J6=$!
    sleep 1
fi

# Node 5
if test $NODECT -ge 5; then
    echo "Starting node 5 ($NODE5_IP): $NODE5_LABEL"
    $XTERM -T "($NODE5_IP) $NODE5_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE5_IP && cat || cat" &
    J5=$!
    sleep 1
fi

# Node 4
if test $NODECT -ge 4; then
    echo "Starting node 4 ($NODE4_IP): $NODE4_LABEL"
    $XTERM -T "($NODE4_IP) $NODE4_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE4_IP && cat || cat" &
    J4=$!
    sleep 1
fi

# Node 3
if test $NODECT -ge 3; then
    echo "Starting node 3 ($NODE3_IP): $NODE3_LABEL"
    $XTERM -T "($NODE3_IP) $NODE3_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE3_IP && cat || cat" &
    J3=$!
    sleep 1
fi

# Node 2
echo "Starting node 2 ($NODE2_IP): $NODE2_LABEL"
$XTERM -T "($NODE2_IP) $NODE2_LABEL" -e $SH -c "$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE2_IP && cat || cat" &
J2=$!

if test "${J10}-${J9}-${J8}-${J7}-${J6}-${J5}-${J4}-${J3}-${J2}" = "--------";  then
    echo "No PIDs for backgrounded JVMs.  Something is wrong."
    exit 1
fi

## Cleanup code to run on trap or at the end of the script:
CLEANUP_CODE="$KILL -9 ${J10} ${J9} ${J8} ${J7} ${J6} ${J5} ${J4} ${J3} ${J2} >/dev/null 2>&1; sleep 1; \
    JVMPIDS=\`$PS | grep \"$ANTS_CONFIGMGR $CONFIG_FILE\" | awk '{print \$1}'\`; \
    test -n \"\$JVMPIDS\" && (echo \"Killing all $CONFIG_FILE-related JVMs...\"; \
	$KILL \$JVMPIDS >/dev/null 2>&1)"

# Kill the other two and exit if anything goes wrong.
trap "echo; echo 'Abort.  Cleaning up...'; $CLEANUP_CODE; exit 1" 1 2 3 13 15

echo "Sleeping for 5 seconds to let nodes get set up..."
sleep 5 || exit 1


# Node 1
echo "Starting node 1 ($NODE1_IP): $NODE1_LABEL"
$JVM $JVMFLAGS $ANTS_CONFIGMGR $CONFIG_FILE $NODE1_IP

# NOTE: uncomment the following line to prevent the code cleanup
# from destroying the xterms before you are ready.
#cat

echo 'Cleaning up...'
eval $CLEANUP_CODE
exit 0

#eof
