/* 
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 */
package ants.core;

/**
 * Assert-like utility functions including <code>assert()</code>,
 * <code>ensure</code> and <code>require()</code>.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class Assert
{
    private static final boolean ENABLED = true;
    
    /**
     * Private constructor prevents instances of this class from
     * being created.
     */
    private Assert()
    {
    }
    
    /**
     * A run-time Error is thrown if an assert fails.
     * All AssertFailures must have a descriptive message.
     */
    public static class AssertFailure
	extends Error
    {
	AssertFailure(String msg)
	{
	    super(msg);
	}
    }
    
    private static void failure(String msg)
    {
	System.out.println("FAILURE:" +msg);
	throw new AssertFailure(msg);
    }
    
    public static void assert(boolean test)
    {
	if (Assert.ENABLED)
	    if (!test)
		failure("assert test");
    }
    
    public static void assert(boolean test, String msg)
    {
	if (Assert.ENABLED)
	    if (!test)
		failure("assert test: " +msg);
    }
    
    public static void assert(Object obj)
    {
	if (Assert.ENABLED)
	    if (obj == null)
		failure("assert <obj> != null");
    }
    
    public static void require(boolean test)
    {
	if (Assert.ENABLED)
	    if (!test)
		failure("require test");
    }
    
    public static void require(Object obj)
    {
	if (Assert.ENABLED)
	    if (obj == null)
		failure("require <obj> != null");
    }
    
    public static void ensure(boolean test)
    {
	if (Assert.ENABLED)
	    if (!test)
		failure("ensure test");
    }
    
    public static void ensure(Object obj)
    {
	if (Assert.ENABLED)
	    if (obj == null)
		failure("ensure <obj> != null");
    }
}
