/*
 * Trace.java
 * 
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 */
package ants.core;

/**
 * Trace class for the various trace macros spread about the ANTS source.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class Trace
{
    public static final boolean ENABLED = true;
    private static final String  DEBUG_PROPERTY = "ants.debugFlags";
    
    public static final int ANTS_STARTUP         = 1<<0;
    public static final int ANTS_DYNLOAD         = 1<<1;
    public static final int ANTS_OUTGOING        = 1<<2;
    public static final int ANTS_CAPSULECHANNEL  = 1<<3;
    public static final int ANTS_APP             = 1<<4;
    public static final int ANTS_CONSOLE         = 1<<5;
    public static final int ANTS_ROUTEINFO       = 1<<6;
    public static final int ANTS_ROUTE_UPDATE    = 1<<7;
    public static final int ANTS_ROUTE_TIMER     = 1<<8;
    public static final int ANTS_ROUTE_READ_TABLE= 1<<9;
    public static final int ANTS_DANTE           = 1<<10;
    public static final int ANTS_NEIGHBOR_TABLE  = 1<<11;
    /* Please update doc/index.html if you add new constants. */
    
    // XXX fragile.  Indicies must line up with bit constants.
    private static final String[] flagNames = {
	"ANTS_STARTUP",
	"ANTS_DYNLOAD",
	"ANTS_OUTGOING",
	"ANTS_CAPSULECHANNEL",
	"ANTS_APP",
	"ANTS_CONSOLE",
	"ANTS_ROUTEINFO",
	"ANTS_ROUTE_UPDATE",
	"ANTS_ROUTE_TIMER",
	"ANTS_ROUTE_READ_TABLE",
	"ANTS_DANTE",
	"ANTS_NEIGHBOR_TABLE"
    };
    
    private static long currentFlags = 0;
    
    static 
    {
	setFlags(System.getProperty(DEBUG_PROPERTY, ""));
    }
    
    public static String[] getNameList()
    {
	if (Trace.ENABLED)
	    return flagNames;
	return null;
    }
    
    public static void setFlags(String flaglist)
    {
	if (Trace.ENABLED)
	{
	    if (flaglist.indexOf("ALL") != -1)
		currentFlags = 0xFfffFfffFfffFfffL;
	    
	    if (flaglist.indexOf("ANTS_ROUTE") != -1)
		currentFlags |= (ANTS_ROUTEINFO|ANTS_ROUTE_UPDATE
				 |ANTS_ROUTE_TIMER|ANTS_ROUTE_READ_TABLE);

	    for (int i = 0; i < flagNames.length; i++)
	    {
		if (flaglist.indexOf(flagNames[i]) != -1) 
		{
		    currentFlags |= (1 << i);
		}
	    }
	}
    }
    
    public static final void trace(int flag, String msg)
    {	  
	
	if (Trace.ENABLED )
	    if ((flag & currentFlags) != 0)
		showMessage(msg);
    }
    
    private static void showMessage(String msg)
    {
	synchronized(Trace.class)
	{
	    System.out.println(msg);
	}
    }
}
