/*
 * Watchdog.Java
 *
 * Copyright (c) 1999, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 */
package ants.core;

import edu.utah.janos.nodeos.Status;

import edu.utah.janos.io.Log;

import java.net.ServerSocket;
import java.net.Socket;

import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.util.Date;

/**
 * A simple watch dog class, it opens a network port and prints out little
 * messages to whoever has connected to it.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
class WatchDog
    implements java.lang.Runnable // XXX
{
    private static final int MAX_SOCKET_BACKLOG = 2;
    
    /**
     * The worker class for the watch dog, this does the actual printing
     * for each connected user.
     */
    private class Worker
	implements Runnable
    {
	private PrintWriter out;
	
	public Worker(PrintWriter pw)
	{
	    this.out = pw;
	}
	
	public void run()
	{
	    while( true )
	    {
		try
		{
		    this.notice();
		}
		catch(java.io.IOException e)
		{
		}
	    }
	}
	
	private void notice()
	    throws java.io.IOException
	{
	    this.out.println("Connected to WatchDog port on "
			     + node);
	    while( true )
	    {
				/* Print out something semi-meaningful */
		this.out.println("** node: "
				 + edu.utah.janos.nodeos.Address.
				 toString(PrimordialNode.
					  getAddress())
				 + " time: "
				 + new Date()
				 + " **");
		this.out.flush();
		try
		{
		    Thread.sleep(1000);
		}
		catch(InterruptedException e)
		{
		}
	    }
	}
    }
    
    /**
     * The socket this watch dog listens on.
     * <p>
     * XXX this should be final but the 1.2.2JDK cannot
     * compile the constructor if it is final.
     */
    private /*final*/ ServerSocket socket;
    
    /**
     * Cache the local Node interface this class should
     * use to start threads.
     * <p>
     * XXX this should be final but the 1.2.2JDK cannot
     * compile the constructor if it is final.
     */
    private /*final*/ Node node;
    
    /**
     * Create a WatchDog listener on the given port.
     * Blows up if the port is already in use, throwing
     * an IOException.
     * <p>
     * Connections will not be handled until the <code>accept()</code>
     * method is executed.  You'd best supply a new thread
     * to handle them, too.
     *
     * @see #accept()
     * @param port the port to use; zero for any free port.
     * @exception IOException throws whatever errors ServerSocket might toss your way.
     */
    WatchDog(Node node, int port)
	throws java.io.IOException // XXX 
    {
	this.node = node;
	
	// May throw IOException:
	this.socket = new ServerSocket(port, MAX_SOCKET_BACKLOG);
    }
    
    /**
     * Return the port this watch dog server listens on
     */
    int getPort()
    {
	return socket.getLocalPort();
    }
    
    public String toString()
    {
	return "WatchDog[" +socket.toString()+ "]";
    }
    
    public void run()
    {
	accept();
    }
    
    /**
     * Accept and handle connections on this watch dog.
     * <p>
     * This call never returns.
     */
    void accept()
    {
	Runnable worker;
	
	while( true )
	{
	    try
	    {
		PrintWriter pw;
		Socket client;
		
		client = this.socket.accept();
		pw = new PrintWriter(client.
				     getOutputStream());
		worker = new Worker(pw);
		client.getInputStream().close();
		this.node.systemThreadStart(worker, "WatchDog worker");
	    }
	    catch(java.io.IOException e)
	    {
	    }
	}
    }
}

