/* 
 * Principal.java
 *
 * Copyright (c) 2001 The University of Washington
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 */
package ants.core.security;

/**
 * Represents a resource principal: something that has access to
 * resources (i.e., administrator, John Smith, etc.).
 * <p>
 * A Principal must be read-only.  Applications must not be allowed to
 * modify fields that are used by the equals or implies method.
 * <p>
 * The implies method is not currently used by the policy classes.
 *
 * @author Andrew Whitaker
 */
public class Principal
    implements java.io.Serializable
{
    private String name;
    
    public Principal(String name)
    {
	this.name = name;
    }
    
    public String getName()
    {
	return name;
    }
    
    /**
     * Case-insensitive equality test.
     */
    public boolean equals(Object foo)
    {
	if (! (foo instanceof Principal))
	    return false;
	else
	{
	    Principal p2 = (Principal) foo;
	    return p2.name.equalsIgnoreCase(this.name);
	}
    }
    
    public int hashCode()
    {
	return name.hashCode();
    }
    
    /**
     * This is currently unused.
     */
    public boolean implies(Principal p)
    {
	return false;
    }
    
    public String toString()
    {
	return "Principal["+name+"]";
    }
    
}
