/* 
 * MobileCapsule.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the ANTS distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to ants-dist@cs.utah.edu
 *         
 *
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 */
package apps.mobile;

import ants.core.*;

/**
 * mobile capsule processing - The mobile capsule is used to carry a payload.
 * Its able to lookup any forwarding information on a router to direct itself
 * properly.
 *
 * @author David Wetherall
 */

public class MobileCapsule
    extends Capsule
{
    final private static byte[] MID = findMID("apps.mobile"
					      + ".MobileCapsule");
    protected byte[] mid()
    {
	return MID;
    }
    
    final private static byte[] PID = findPID("apps.mobile"
					      + ".MobileCapsule");
    protected byte[] pid()
    {
	return PID;
    }
    
    /**
     * The next destination for the capsule.  First, points to the original
     * destination, and then to a forwarding address if there is one.
     */
    int next;
    
    /**
     * Source and destination port...
     */
    short spt;
    short dpt;
    
    /**
     * The data we're sending
     */
    ByteArray data;
    
    public int length()
    {
	int s = Xdr.INT +
	    Xdr.SHORT +
	    Xdr.SHORT +
	    Xdr.BYTEARRAY(this.data);
	return super.length() + s;
    }
    
    
    public Xdr encode()
    {
	Xdr xdr = super.encode();
	
	xdr.PUT(this.next);
	xdr.PUT(this.spt);
	xdr.PUT(this.dpt);
	xdr.PUT(this.data);
	
	return xdr;
    }
    
    public Xdr decode()
    {
	Xdr xdr = super.decode();
	
	this.next = xdr.INT();
	this.spt = xdr.SHORT();
	this.dpt = xdr.SHORT();
	this.data = xdr.BYTEARRAY();
	
	return xdr;
    }
    
    /**
     * Check for a forwarding address and then ship the packet
     */
    public boolean evaluate(Node n)
    {
	boolean retval;
	
	if (n.getAddress() == getDst())
	{
	    /* We're at the destination, pass it to the app */
	    retval = n.deliverToApp(this, this.dpt);
	}
	else
	{
	    /* Check the cache for a forwarding address */
	    Integer f = (Integer)n.getCache().get(this.getDst());
	    
	    /*
	     * If theres a forwarding address then set it as our
	     * next place to send the packet
	     */
	    if (f != null)
		this.next = f.intValue();
	    retval = n.routeForNode(this, this.next);
	}
	return retval;
    }
    
    public MobileCapsule()
    {
    }
    
    public MobileCapsule(int na, short sa, short da, ByteArray d)
    {
	/* We send to the original destination first */
	setDst(na);
	this.next = na;
	this.spt = sa;
	this.dpt = da;
	this.data = d;
    }
    
}
